@@ -59,9 +59,14 @@
 #'   \code{tools::\link[tools]{package_dependencies}()} for possible values. The
 #'   special value \code{'hard'} means the hard dependencies, i.e.,
 #'   \code{c('Depends', 'Imports', 'LinkingTo')}.
-#' @param recheck Whether to only check the failed packages from last time. By
-#'   default, if there are any \file{*.Rcheck} directories, \code{recheck} will
-#'   be automatically set to \code{TRUE} if missing.
+#' @param recheck A vector of package names to be (re)checked. If not provided
+#'   and there are any \file{*.Rcheck} directories left by certain packages
+#'   (this often means these packages failed the last time), \code{recheck} will
+#'   be these packages; if there are no \file{*.Rcheck} directories but a text
+#'   file \file{recheck} exists, \code{recheck} will be the character vector
+#'   read from this file. This provides a way for you to manually specify the
+#'   packages to be checked. If there are no packages to be rechecked, all
+#'   reverse dependencies will be checked.
 #' @param ignore A vector of package names to be ignored in \command{R CMD
 #'   check}. If this argument is missing and a file \file{00ignore} exists, the
 #'   file will be read as a character vector and passed to this argument.
@@ -70,6 +75,7 @@
 #' @param src_dir The parent directory of the source package directory. This can
 #'   be set in a global option if all your source packages are under a common
 #'   parent directory.
+#' @param timeout Timeout in seconds for \command{R CMD check}.
 #' @seealso \code{devtools::revdep_check()} is more sophisticated, but currently
 #'   has a few major issues that affect me: (1) It always deletes the
 #'   \file{*.Rcheck} directories
@@ -89,7 +95,8 @@
 #'   dependencies.
 #' @export
 rev_check = function(
-  pkg, which = 'all', recheck = FALSE, ignore = NULL, update = TRUE,
+  pkg, which = 'all', recheck = NULL, ignore = NULL, update = TRUE,
+  timeout = getOption('xfun.rev_check.timeout', 15 * 60),
   src = file.path(src_dir, pkg), src_dir = getOption('xfun.rev_check.src_dir')
 ) {
   if (length(src) != 1 || !dir.exists(src)) stop(
@@ -102,11 +109,14 @@ rev_check = function(
   db = available.packages(type = 'source')
 
   unlink('*.Rcheck2', recursive = TRUE)
-  dirs = list.files('.', '.+[.]Rcheck$')
-  if (missing(recheck)) recheck = length(dirs) > 0
-  pkgs = if (recheck) {
-    gsub('.Rcheck$', '', dirs)
-  } else {
+  if (missing(recheck)) {
+    dirs = list.files('.', '.+[.]Rcheck$')
+    pkgs = gsub('.Rcheck$', '', dirs)
+    recheck = if (length(pkgs) == 0 && file.exists('recheck')) {
+      scan('recheck', 'character')
+    } else pkgs
+  }
+  pkgs = if (length(recheck)) recheck else {
     res = check_deps(pkg, db, which)
     pkgs_up = NULL
     if (update) {
@@ -178,7 +188,8 @@ rev_check = function(
     check_it = function(args = NULL, ...) {
       system2(
         file.path(R.home('bin'), 'R'),
-        c(args, 'CMD', 'check', '--no-manual', shQuote(z)), stdout = FALSE, stderr = FALSE, ...
+        c(args, 'CMD', 'check', '--no-manual', shQuote(z)),
+        stdout = FALSE, stderr = FALSE, timeout = timeout, ...
       )
     }
     check_it()
@@ -190,14 +201,24 @@ rev_check = function(
         file.path(d, 'vign_test', p, 'vignettes'), '[.](Rnw|Rmd)',
         ignore.case = TRUE, full.names = TRUE
       )
+      pkg_load2('tinytex')
       if (length(vigs) && any(file.exists(with_ext(vigs, 'log')))) {
-        if (!loadable('tinytex')) install.packages('tinytex')
         if (tinytex:::is_tinytex()) for (vig in vigs) in_dir(dirname(vig), {
           Rscript(shQuote(c('-e', 'if (grepl("[.]Rnw$", f <- commandArgs(T), ignore.case = T)) knitr::knit2pdf(f) else rmarkdown::render(f)', basename(vig))))
         })
         check_it()
         if (clean_Rcheck(d)) return(timing())
       }
+      # if there are still missing LaTeX packages, install them and recheck
+      l0 = tinytex::tl_pkgs()
+      lapply(
+        list.files(d, '[.]log$', full.names = TRUE, recursive = TRUE),
+        tinytex::parse_install, quiet = TRUE
+      )
+      if (!identical(l0, tinytex::tl_pkgs())) {
+        check_it()
+        if (clean_Rcheck(d)) return(timing())
+      }
       # clean up the check log, and recheck with the current CRAN version of pkg
       cleanup = function() in_dir(d, {
         clean_log()
@@ -305,6 +326,12 @@ download_tarball = function(p, db = available.packages(type = 'source'), dir = '
 # option xfun.install.packages
 pkg_install = function(pkgs, ...) {
   if (length(pkgs) == 0) return()
+  # in case the CRAN repo is not set up
+  repos = getOption('repos')
+  if (length(repos) == 0 || identical(repos, c(CRAN = '@CRAN@'))) {
+    opts = options(repos = c(CRAN = 'https://cran.rstudio.com'))
+    on.exit(options(opts), add = TRUE)
+  }
   install = getOption('xfun.install.packages', install.packages)
   if (length(pkgs) > 1)
     message('Installing ', length(pkgs), ' packages: ', paste(pkgs, collapse = ' '))
@@ -410,14 +437,14 @@ cran_check_pages = function() {
   }
 }
 
-# kill a R CMD check process if it has been running for more then 60 minutes
+# kill a R CMD check process if it has been running for more then 30 minutes
 kill_long_processes = function(etime = 30) {
   while (TRUE) {
     if (length(pids <- list_long_processes(etime))) {
       message('Killing processes: ', paste(pids, collapse = ' '))
       system2('kill', pids)
     }
-    Sys.sleep(300)
+    Sys.sleep(30)
   }
 }
 
@@ -445,8 +472,8 @@ install_missing_latex = function() {
   tinytex::tlmgr_install(unique(pkgs))
 }
 
-# return packages that haven't been updated for X days, and can be updated on CRAN
-cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
+# retrieve the release dates of packages
+cran_pkg_dates = function(full = FALSE, maintainer = 'Yihui Xie') {
   info = tools::CRAN_package_db()
   pkgs = info[grep(maintainer, info$Maintainer), 'Package']
   info = setNames(vector('list', length(pkgs)), pkgs)
@@ -464,6 +491,12 @@ cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
     d = c(d, as.Date(gsub(r, '\\1', grep(r, x, value = TRUE))))
     info[[p]] = sort(d, decreasing = TRUE)
   }
+  if (full) info else sort(do.call(c, lapply(info, `[`, 1)), decreasing = TRUE)
+}
+
+# return packages that haven't been updated for X days, and can be updated on CRAN
+cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
+  info = cran_pkg_dates(TRUE, maintainer)
   flag = unlist(lapply(info, function(d) {
     sum(d > Sys.Date() - 180) < 6 && d[1] < Sys.Date() - days
   }))
